<?php
defined('BASEPATH') or exit('No direct script access allowed');

class Product_items_control extends CI_Controller
{
  public function __construct()
  {
    parent::__construct();

    // Ensure the user is an admin
    if ($this->session->userdata('prosc_admin') !== true) {
      redirect(base_url(), 'refresh');
    }

    // Load required models
    $this->load->model('product_items_model');
    $this->load->model('dashboard_model');
    $this->load->model('shipping_settings_model');
    $this->load->model('notification_model'); // Adding this explicitly since it's used in the original code
  }

  /**
   * Default method to load the main page
   */
  public function index()
  {
    $this->pager();
  }

  /**
   * Centralized method to return JSON responses
   */
  private function jsonResponse($status, $message, $extra = [])
  {
    echo json_encode(array_merge(['status' => $status, 'message' => $message], $extra));
    exit;
  }

  /**
   * Set minimum stock level
   */
  public function stock()
  {
    $value = trim($this->input->post('value'));

    // Validate stock value
    if (!is_numeric($value) || $value < 1) {
      $this->jsonResponse('error', 'Minimum Inventory Stock cannot be less than 1');
    }

    // Update stock in the database
    $this->product_items_model->_set_min($value);
    $this->session->set_flashdata('prosc_message', "Minimum Inventory Stock is now set to $value");
    $this->jsonResponse('success', '', ['extra' => '']);
  }

  /**
   * Load products and related data
   */
  public function pager($tmp = '')
  {
    // Restrict direct access
    if (!$this->input->is_ajax_request() && strstr(current_url(), '.aspx') == '') {
      redirect(base_url('not_found.aspx'), 'refresh');
    }

    // Fetch necessary data
    $data = $this->product_items_model->_get_all() ?: [];
    $categories = $this->product_items_model->_get_categories() ?: [];
    $country = $this->shipping_settings_model->_get_country();
    $states = ($country->value === 'Nigeria') ?
      $this->shipping_settings_model->_get_nigeria() :
      $this->shipping_settings_model->_get_uk();
    $currency = $this->dashboard_model->_get_currency() ?: "";
    $min_stock = $this->notification_model->_get_min() ?: 1;

    // Load the appropriate view
    if ($tmp == '') {
      $this->load->view('default', [
        'page' => 'product_items',
        'data' => $data,
        'categories' => $categories,
        'currency' => $currency,
        'states' => $states,
        'tmp' => $tmp,
        'stock' => $min_stock
      ]);
    } else {
      return $this->load->view('admin/product_items', [
        'data' => $data,
        'categories' => $categories,
        'states' => $states,
        'currency' => $currency,
        'tmp' => $tmp
      ], true);
    }
  }

  /**
   * Validate and prepare product data
   */
  private function validateProductData()
  {
    $name = trim($this->input->post('name'));
    $unit = trim($this->input->post('unit'));
    $price = intval(trim($this->input->post('price')));
    $cost_price = intval(trim($this->input->post('cost_price')));
    $category = trim($this->input->post('category'));
    $available_online = trim($this->input->post('available_online'));
    $expiry_date = trim($this->input->post('expiry_date'));
    $description = trim($this->input->post('description'));
    $batch_number = trim($this->input->post('batch_number'));
    $image = $this->input->post('image');
    $min_stock = $this->input->post('min_stock');
    $discount = $this->input->post('discount');
    $stock = $this->input->post('stock');

    // Basic validations
    if (empty($name) || !is_string($name) || is_numeric($name)) {
      return 'Product name must be a valid string.';
    }
    if (!is_numeric($price) || $price < 1) {
      return 'Product price must be a number greater than zero.';
    }
    if (!is_numeric($cost_price) || $cost_price < 1) {
      return 'Product cost price must be a number greater than zero.';
    }

    // Prepare data
    $data = [
      'name' => $name,
      'description' => $description,
      'unit' => $unit,
      'price' => json_encode(['a577f519b58b533e400cf28a7464576babf92121' => [$price, $cost_price]]),
      'category' => $category,
      'discount' => $discount,
      'available_online' => $available_online,
      'batch_number' => $batch_number,
      'expiry_date' => $expiry_date,
      'vat' => 0,
      'stock' => $stock,
      'min_stock' => $min_stock,
      'done_by' => $this->session->userdata('prosc_admin')
    ];

    if ($image) {
      $data['image'] = base64_decode($image);
    }

    return $data;
  }

  /**
   * Add a new product
   */
  public function create()
  {
    $data = $this->validateProductData();
    if (!is_array($data)) {
      $this->jsonResponse('error', $data);
    }

    if ($this->product_items_model->_get(['name' => $data['name'], 'unit' => $data['unit'], 'category' => $data['category']])) {
      $this->jsonResponse('error', 'A similar item already exists in inventory.');
    }

    if (!$this->product_items_model->_post($data)) {
      $this->jsonResponse('error', 'An unknown error occurred and the insertion operation failed.');
    }

    $this->jsonResponse('success', $this->pager('dump'));
  }

  /**
   * Update an existing product
   */
  public function update()
  {
    $target = trim($this->input->get('target'));
    if (!$this->product_items_model->_get(['token' => $target])) {
      $this->jsonResponse('error', 'A fatal error has occurred. <br />Please refresh and try again.');
    }

    $data = $this->validateProductData();
    if (!is_array($data)) {
      $this->jsonResponse('error', $data);
    }

    if ($this->product_items_model->_get(['token !=' => $target, 'name' => $data['name'], 'unit' => $data['unit'], 'category' => $data['category']])) {
      $this->jsonResponse('error', 'A similar item already exists in inventory.');
    }

    if (!$this->product_items_model->_put($data, $target)) {
      $this->jsonResponse('error', 'An unknown error occurred and the update failed.');
    }

    $this->jsonResponse('success', base_url("ipanel/product_items.aspx"));
  }

  /**
   * Delete a product
   */
  public function delete()
  {
    $target = trim($this->input->get('token'));
    if (!$this->product_items_model->_get(['token' => $target])) {
      $this->jsonResponse('error', 'A fatal error has occurred. <br />Please refresh and try again.');
    }

    $this->product_items_model->_drop($target);
    $this->jsonResponse('success', $this->pager('dump'));
  }

  /**
   * Display popup with product details
   */
  public function popup()
  {
    if (!$this->input->is_ajax_request()) {
      $pagedata['heading'] = 'Hold on!';
      $pagedata['message'] = 'The page you tried to reach restricts direct access.';
      $this->load->view('errors/html/error_404', $pagedata);
    }

    $last = $this->product_items_model->_count() + 1;
    $target = $this->input->get('token');
    $data = ['data' => '', 'last' => $last];
    $data['categories'] = $this->product_items_model->_get_categories() ?: [];
    $file = $this->product_items_model->_get(['token' => $target]);
    $states = $this->product_items_model->_get_states();

    if ($file) $data['data'] = $file;
    $data['states'] = $states;

    $this->load->view('popups/admin/items', $data);
  }
}
